<?php

/**
 * @file
 * GUI Component abstract classed used to create custom components.
 * @author TJ Holowaychuk <tj@vision-media.ca>
 * @link http://vision-media.ca
 * @package gui
 */

/**
 * GUI_Component
 *
 * A GUI Component object is the basis of what becomes an interactive
 * graphical representation of an interface such as tabs, accordions, datagrids
 * and much more. Although you cannot instance this class you must extend it
 * when creating your own components.
 *
 * HOOKS
 *
 * The GUI_Component class issues 'hooks' in a similar manor to regular Drupal
 * hooks, however they are methods. Currently the following hooks may be implemented:
 *   - hook::add_js()
 *     Return an array of javascript filenames relative to your component module. This 
 *     is handled in this manor to allow for future Ajax instances of components to be 
 *     created.
 *
 *   - hook::add_css() 
 *     Similar to to hook::add_js() however this allows components to provide base 
 *     css structures, which then are most likely enhanced with graphics and colors
 *     using a provided skin.
 *
 *   - hook::add_settings()
 *     An associative array of settings visible to javascript. Use in place of drupal_add_js().
 *     These settings will be available from the 'settings' property of your javascript component
 *     implementation. 
 *
 * NOTES
 * 
 * When implementing a constructor function for your class you should call the parent method which
 * can be done using parent::__construct(). This ensures that GUI_Component may still perform its
 * original processing (be sure to pass $component_info to the parent as well).
 * 
 * The entire concept of the GUI_Component implementations is to essentially provide very clean
 * and simple methods of creating such interactive components. For example the tabs component may 
 * provide several methods useful for customizing and adding tabs, or a context sensative menu
 * component may allow you to add seperators or menu items.
 */
abstract class GUI_Component {
	
	/**
	 * Id.
	 *
	 * @var int
	 */
	public $id = 0;
	
	/**
	 * JavaScript filepaths.
	 *
	 * @var array
	 */
	public $js = array();
	
	/**
	 * JavaScript settings.
	 *
	 * @var array
	 */
	public $settings = array();
		
	/**
	 * CSS filepaths.
	 *
	 * @var array
	 */
	public $css = array();
	
	/**
	 * Implementation component info.
	 *
	 * @var array
	 */
	public $component_info = array();
	
	/**
	 * Implementation skin info.
	 *
	 * @var array
	 */
	public $skin_info = array();
	
	/**
	 * Current theme choosen.
	 *
	 * @var string
	 */
	public $current_skin = 'default';
	
	/**
	 * Module path.
	 *
	 * @var string
	 */
	public $module_path = '';
			
  /**
   * Constructor.
   */
  public function __construct($component_info) {
	  $this->component_info = $component_info;
	  $this->skin_info = $this->get_skin_info();
	  $this->current_skin = $this->get_current_skin();
	  $this->module_path = drupal_get_path('module', $this->component_info['module']);
    $this->invoke_hooks(); 
  }

  /* -----------------------------------------------------------------

    Public Methods

  ------------------------------------------------------------------ */

  /**
   * Initialize.
   */
  public function init() {
	  $this->_add_js();
    $this->_add_css();
    $this->_add_settings();
  }

  /**
   * Invoke all hooks.
   */
  public function invoke_hooks() {
    $hooks = array('add_js' => 'js', 'add_css' => 'css', 'add_settings' => 'settings');
    foreach((array) $hooks AS $hook => $prop) {
	    if (method_exists($this, $hook)){
		    $this->$prop = $this->$hook();
	    }
    }
  }

  /**
   * Set skin information.
   * 
   * @return array
   */
  public function get_skin_info() {
    $skins = gui_invoke_skins();
    return $skins[$this->component_info['name']];
  }

  /**
   * Get current skin info.
   * 
   * @return array
   */
  public function get_current_skin() {
    // @todo implement settings 
    return $this->skin_info[variable_get('gui_component_tabs_skin', $this->component_info['default skin'])];
  }

  /* -----------------------------------------------------------------

    Private Methods

  ------------------------------------------------------------------ */

  /**
   * Add JavaSripts.
   */
  private function _add_js() {
    if (!empty($this->js)){ 
	    foreach((array) $this->js AS $filepath){ 
		    drupal_add_js($this->module_path . '/' . ltrim($filepath, '/'));
	    }
    }
  }

  /**
   * Add CSS.
   */
  private function _add_css() {
    if (!empty($this->css)){
	    foreach((array) $this->css AS $filepath){
		    drupal_add_css($this->module_path . '/' . ltrim($filepath, '/'));
	    }
    }
  }

  /**
   * Add JavaScript settings.
   */
  private function _add_settings() {
    drupal_add_js(array('gui' => array($this->component_info['name'] => $this->settings)), 'setting');
  }  
}
